"""Util that calls MultiOn Client.

In order to set this up, follow instructions at:
https://multion.notion.site/Download-MultiOn-ddddcfe719f94ab182107ca2612c07a5
"""
from typing import Any, Optional

from pydantic import BaseModel


class MultionAPI:
    def __init__(self) -> None:
        self.tabId = None
        self.new_session_count = 0

    def create_session(self, query: str, url: str) -> str:
        """Always the first step to run any activities that can be done using browser.

        Args:
            'query': the query that you need to perform in the given url.
                If there is no 'query' set it as open.
            'url': the base url of a site.
        """
        import multion

        # Only create new session once and continue using update session
        if self.new_session_count < 2:
            response = multion.new_session({"input": query, "url": url})
            self.new_session_count += 1
            self.tabId = response["tabId"]
            return response["message"]
        else:
            return "Continue using update session"

    def update_session(self, query: str, url: str) -> str:
        """Updates the existing browser session.

        Updates with given action and url, used consequently to handle browser
        activities after creating one session of browser.

        Args:
            'query': the query that you need to perform in the given url.
                If there is no 'query' set it as open.
            'url': the base url of a site.
        """
        import multion

        response = multion.update_session(self.tabId, {"input": query, "url": url})
        return response["message"]


class MultionClientAPIWrapper(BaseModel):
    """Wrapper for Multion Client API.

    In order to set this up, follow instructions at:
    NEED TO ADD
    """

    client: Any = MultionAPI()

    def run(self, task: str, url: str, tabId: Optional[Any]) -> str:
        """Run body through Multion Client and respond with action.

        Args:
            task:
            url:
            tabId:
        """
        if self.client.tabId is None or tabId is None:
            self.client = MultionAPI()
            message = self.client.create_session(task, url)
        else:
            message = self.client.update_session(task, url)

        return message



"""Tool for MultiOn Extension API"""
from typing import Any, Optional

from pydantic import Field

from langchain.callbacks.manager import CallbackManagerForToolRun
from langchain.tools.base import BaseTool
from langchain.utilities.multion import MultionClientAPIWrapper


def _get_default_multion_client() -> MultionClientAPIWrapper:
    return MultionClientAPIWrapper()


class MultionClientTool(BaseTool):
    """Simulates a Browser interacting agent."""

    name = "Multion_Client"
    description = (
        "A api to communicate with browser extension multion "
        "Useful for automating tasks and actions in the browser "
        "Input should be a task and a url."
        "The result is text form of action that was executed in the given url."
    )
    api_wrapper: MultionClientAPIWrapper = Field(
        default_factory=_get_default_multion_client
    )

    def _run(
        self,
        task: str,
        url: str = "https://www.google.com/",
        tabId: Optional[Any] = None,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> str:
        """Use the tool."""
        return self.api_wrapper.run(task, url, tabId)
